recon_ecm = false
local tmp_vec1 = Vector3()
ECMJammerBase = ECMJammerBase or class(UnitBase)
ECMJammerBase._NET_EVENTS = {
	battery_low = 1,
	battery_empty = 2,
	feedback_start = 3,
	feedback_stop = 4,
	jammer_active = 5,
	feedback_flash = 6,
	feedback_restart = 7
}
ECMJammerBase.battery_life_multiplier = {
	1,
	1.25,
	1.5
}

function ECMJammerBase:init(unit) -- Need to mod this
	UnitBase.init(self, unit, true)
	self._unit = unit
	self._position = self._unit:position()
	self._rotation = self._unit:rotation()
	self._g_glow_jammer_green = self._unit:get_object(Idstring("g_glow_func1_green"))
	self._g_glow_jammer_red = self._unit:get_object(Idstring("g_glow_func1_red"))
	self._g_glow_feedback_green = self._unit:get_object(Idstring("g_glow_func2_green"))
	self._g_glow_feedback_red = self._unit:get_object(Idstring("g_glow_func2_red"))
	self._max_battery_life = tweak_data.upgrades.ecm_jammer_base_battery_life
	self._battery_life = self._max_battery_life
	self._low_battery_life = tweak_data.upgrades.ecm_jammer_base_low_battery_life
	self._feedback_active = false
	self._jammer_active = false
	-- Check if skill on
	if managers.player:has_category_upgrade("ecm_jammer", "recon_enabled") then
		log('recon_ENABLED!')
		recon_ecm = true
	end
	if Network:is_client() then
		self._validate_clbk_id = "ecm_jammer_validate" .. tostring(unit:key())
		managers.enemy:add_delayed_clbk(self._validate_clbk_id, callback(self, self, "_clbk_validate"), Application:time() + 60)
	end
end

function ECMJammerBase:sync_net_event(event_id)
	local net_events = self._NET_EVENTS
	if event_id == net_events.battery_low then
		self:_set_battery_low()
	elseif event_id == net_events.battery_empty then
		self:_set_battery_empty()
	elseif event_id == net_events.feedback_start then
		self:_set_feedback_active(true)
		-- Range mod as client here
		-- loud check managers.groupai:state():whisper_mode()
		local range = tweak_data.upgrades.ecm_jammer_base_range
		local detected = World:find_units_quick("sphere", self._unit:position(), range, managers.slot:get_mask("enemies"))
		for _, unit in ipairs(detected) do
			log('AS_CLIENT_recon_ENABLED:'..tostring(recon_ecm))
			if alive(unit:body("mover_blocker")) and recon_ecm then
				--Mod here
				unit:contour():add("mark_enemy_damage_bonus", true, 2)
			end
		end
	elseif event_id == net_events.feedback_stop then
		self:_set_feedback_active(false)
	elseif event_id == net_events.jammer_active then
		self:set_active(true)
	elseif event_id == net_events.feedback_flash then
		self._unit:contour():flash("deployable_active", 0.15)
	elseif event_id == net_events.feedback_restart then
		self._unit:sound_source():post_event("ecm_jammer_ready")
	end
end

function ECMJammerBase._detect_and_give_dmg(hit_pos, device_unit, user_unit, range)
	local mvec3_dis_sq = mvector3.distance_sq
	local slotmask = managers.slot:get_mask("bullet_impact_targets")
	local splinters = {
		mvector3.copy(hit_pos)
	}
	local dirs = {
		Vector3(range, 0, 0),
		Vector3(-range, 0, 0),
		Vector3(0, range, 0),
		Vector3(0, -range, 0),
		Vector3(0, 0, range),
		Vector3(0, 0, -range)
	}
	local pos = tmp_vec1
	for _, dir in ipairs(dirs) do
		mvector3.set(pos, dir)
		mvector3.add(pos, hit_pos)
		local splinter_ray = World:raycast("ray", hit_pos, pos, "slot_mask", slotmask)
		pos = (splinter_ray and splinter_ray.position or pos) - dir:normalized() * math.min(splinter_ray and splinter_ray.distance or 0, 10)
		local near_splinter = false
		for _, s_pos in ipairs(splinters) do
			if mvector3.distance_sq(pos, s_pos) < 300 then
				near_splinter = true
			else
			end
		end
		if not near_splinter then
			table.insert(splinters, mvector3.copy(pos))
		end
	end
	local range_sq = range * range
	local half_range_sq = range * 0.5
	half_range_sq = half_range_sq * half_range_sq
	local function _chk_apply_dmg_to_char(u_data)
		if not u_data.char_tweak.ecm_vulnerability then
			return
		end
		if u_data.unit.brain and u_data.unit:brain().is_hostage and u_data.unit:brain():is_hostage() then
			return
		end
		if u_data.unit:anim_data() and u_data.unit:anim_data().act then
			return
		end
		if math.random() >= u_data.char_tweak.ecm_vulnerability then
			return
		end
		local head_pos = u_data.unit:movement():m_head_pos()
		local dis_sq = mvec3_dis_sq(head_pos, hit_pos)
		if dis_sq > range_sq then
			return
		end
		for i_splinter, s_pos in ipairs(splinters) do
			local ray_hit = World:raycast("ray", s_pos, head_pos, "slot_mask", slotmask, "ignore_unit", u_data.unit, "report")
			if not ray_hit and (i_splinter == 1 or dis_sq < half_range_sq) then
				local attack_data = {
					variant = "stun",
					damage = 0,
					attacker_unit = alive(user_unit) and user_unit or nil,
					weapon_unit = device_unit,
					col_ray = {
						position = mvector3.copy(head_pos),
						ray = head_pos - s_pos:normalized()
					}
				}
				log('AS_HOST_recon_ENABLED:'..tostring(recon_ecm))
				if alive(u_data.unit:body("mover_blocker")) and recon_ecm then
					log('MARKED')
					u_data.unit:contour():add("mark_enemy_damage_bonus", true, 2)
				end
				u_data.unit:character_damage():damage_explosion(attack_data)
			else
			end
		end
	end
	for u_key, u_data in pairs(managers.enemy:all_enemies()) do
		_chk_apply_dmg_to_char(u_data)
	end
	for u_key, u_data in pairs(managers.enemy:all_civilians()) do
		_chk_apply_dmg_to_char(u_data)
	end
end

